import pytest
import numpy as np
from rq_controller.common import PyRegionOfIntrest  # Adjust this import according to your module's path
from rq_interfaces.msg import RegionOfIntrest
from visualization_msgs.msg import Marker

@pytest.fixture
def example_message():
    msg = RegionOfIntrest()
    
    marker = Marker()
    marker.pose.position.x = 10.0
    marker.pose.position.y = 20.0
    marker.pose.position.z = 30.0
    marker.scale.x = 40.0
    marker.scale.y = 50.0
    marker.scale.z = 60.0
    marker.header.frame_id = "test_frame"
    
    msg.region_of_intrest_stack.markers.append(marker)
    
    msg.resolution.x = 0.1
    msg.resolution.y = 0.1
    msg.resolution.z = 0.1
    
    return msg

def test_initialization():
    center_points_mm = np.array([[1.0, 2.0, 3.0]])
    dimensions_mm = np.array([[4.0, 5.0, 6.0]])
    resolution_mm = np.array([[0.1, 0.1, 0.1]])
    frame_id = "test_frame"

    roi = PyRegionOfIntrest(center_points_mm, dimensions_mm, frame_id, resolution_mm)

    assert np.array_equal(roi.center_points_mm, center_points_mm)
    assert np.array_equal(roi.dimensions_mm, dimensions_mm)
    assert np.array_equal(roi.resolution_mm, resolution_mm)
    assert roi.frame_id == frame_id

def test_dummy_method():
    roi = PyRegionOfIntrest.dummy()

    assert roi.center_points_mm.shape == (1, 3)
    assert roi.dimensions_mm.shape == (1, 3)
    assert roi.frame_id == "object"

def test_from_message(example_message):
    roi = PyRegionOfIntrest.from_message(example_message)

    assert np.array_equal(roi.center_points_mm, np.array([[10.0, 20.0, 30.0]]))
    assert np.array_equal(roi.dimensions_mm, np.array([[40.0, 50.0, 60.0]]))
    assert np.array_equal(roi.resolution_mm, np.array([[0.1, 0.1, 0.1]]))
    assert roi.frame_id == "test_frame"

def test_as_message(example_message):
    roi = PyRegionOfIntrest.from_message(example_message)
    msg = roi.as_message()

    assert len(msg.region_of_intrest_stack.markers) == 1
    marker = msg.region_of_intrest_stack.markers[0]

    assert marker.pose.position.x == 10.0
    assert marker.pose.position.y == 20.0
    assert marker.pose.position.z == 30.0
    assert marker.scale.x == 40.0
    assert marker.scale.y == 50.0
    assert marker.scale.z == 60.0
    assert marker.header.frame_id == "test_frame"
    
    assert msg.resolution.x == 0.1
    assert msg.resolution.y == 0.1
    assert msg.resolution.z == 0.1

def test_number_of_rois(example_message):
    roi = PyRegionOfIntrest.from_message(example_message)
    assert roi.number_of_rois == 1

def test_shape(example_message):
    roi = PyRegionOfIntrest.from_message(example_message)
    assert roi.shape == (399, 499, 599)

def test_get_grid(example_message):
    roi = PyRegionOfIntrest.from_message(example_message)
    grid = roi.get_grid(0)

    assert grid.shape == (399, 499, 599, 3)
    assert np.array_equal(grid[0, 0, 0], np.array([10.0 - 20.0, 20.0 - 25.0, 30.0 - 30.0]))
    assert np.array_equal(grid[-1, -1, -1], np.array([10.0 + 20.0, 20.0 + 25.0, 30.0 + 30.0]))

def test_next_neighbor():
    grid_mm = np.zeros((20, 30, 40, 3))
    grid_mm[:, :, :, 0] = np.linspace(0, 20, 20).reshape(-1, 1, 1)
    grid_mm[:, :, :, 1] = np.linspace(0, 30, 30).reshape(1, -1, 1)
    grid_mm[:, :, :, 2] = np.linspace(0, 40, 40).reshape(1, 1, -1)

    point_mm = np.array([11.0, 16.0, 21.0])
    index = PyRegionOfIntrest.next_neighbor(grid_mm, point_mm)

    assert np.array_equal(index, [10, 15, 20])
